package edujin.handler;

import com.sun.net.httpserver.HttpExchange;
import edujin.model.Logging;
import edujin.repository.LoggingRepository;
import io.github.cdimascio.dotenv.Dotenv;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import javax.xml.namespace.QName;
import javax.xml.soap.*;
import javax.xml.ws.handler.MessageContext;
import javax.xml.ws.handler.soap.SOAPHandler;
import javax.xml.ws.handler.soap.SOAPMessageContext;
import java.util.Set;

public class LoggingHandler implements SOAPHandler<SOAPMessageContext> {
    private final String REST_API_KEY = Dotenv.load().get("REST_KEY");
    private final String APP_API_KEY = Dotenv.load().get("APP_KEY");
    private final String httpExchangeKey = "com.sun.xml.internal.ws.http.exchange";

    private String source;

    @Override
    public Set<QName> getHeaders() {

        return null;
    }

    @Override
    public boolean handleMessage(SOAPMessageContext context) {
        Boolean outboundProperty = (Boolean) context.get(MessageContext.MESSAGE_OUTBOUND_PROPERTY);
        if (outboundProperty) {
            return true;
        }
        String clientApiKey = getClientApiKey(context);
        if (clientApiKey == null) {
            return false;
        }
        if (clientApiKey.equals(REST_API_KEY)) {
            source = "REST";
        } else if  (clientApiKey.equals(APP_API_KEY)) {
            source = "SOAP";
        } else {
            return false;
        }

        try {
            SOAPBody soapBody = context.getMessage().getSOAPBody();
            String s = "Logging: " + getClientRequest(soapBody) + "(" + getClientParamter(soapBody) + ")";
            String description = getClientRequest(soapBody) + "("+ getClientParamter(soapBody) + ")";
            Logging logging = new Logging();

            System.out.println("Description: " + description);
            logging.setRequest(description);
            logging.setIP(getClientIp(context));
            logging.setAPIKey(clientApiKey);
            logging.setSource(source);
            logging.setEndpoint(getHTTPRequest(context) + " " + getEndpoint(context));
            LoggingRepository.getInstance().create(logging);
        } catch (SOAPException e) {
            throw new RuntimeException(e);
        }
        return true;
    }

    @Override
    public boolean handleFault(SOAPMessageContext context) {
        return false;
    }

    @Override
    public void close(MessageContext context) {

    }
    protected String getClientIp(SOAPMessageContext smc) {
        HttpExchange exchange = (HttpExchange) smc.get(httpExchangeKey);
        System.out.println("Client IP" + exchange.getRemoteAddress().getAddress().getHostAddress());
        return exchange.getRemoteAddress().getAddress().getHostAddress();
    }
    protected String getClientApiKey(SOAPMessageContext smc) {
        HttpExchange exchange = (HttpExchange) smc.get(httpExchangeKey);
        System.out.println("Client API KEY " + exchange.getRequestHeaders().getFirst("X-API-KEY"));
        return exchange.getRequestHeaders().getFirst("X-API-KEY");
    }
    protected String getHTTPRequest(SOAPMessageContext smc) {
        HttpExchange exchange = (HttpExchange) smc.get(httpExchangeKey);
        System.out.println("HTTP Request " + exchange.getRequestMethod());
        return exchange.getRequestMethod();
    }
    protected String getEndpoint(SOAPMessageContext smc) {
        HttpExchange exchange = (HttpExchange) smc.get(httpExchangeKey);
        System.out.println("Endpoint " + exchange.getRequestURI().toString());
        return exchange.getRequestURI().toString();
    }
    protected String getClientRequest(SOAPBody soapBody) throws SOAPException {
        Node firstChildNode = soapBody.getFirstChild().getNextSibling();
        return firstChildNode.getNodeName();
    }

    protected String getClientParamter(SOAPBody soapBody) throws SOAPException {
        Node firstChildNode = soapBody.getFirstChild().getNextSibling();

        StringBuilder parameters = new StringBuilder();
        NodeList nodeList = firstChildNode.getChildNodes();
        int parameterCount = nodeList.getLength();
        for (int i = 0; i < parameterCount - 2; i++) {
            Node node = nodeList.item(i);
            if (node instanceof SOAPElement) {
                SOAPElement element = (SOAPElement) node;
                String paramName = element.getLocalName();
                String paramValue = element.getValue();
                parameters.append(paramName).append("=").append(paramValue).append(", ");
            }
        }
        Node node = nodeList.item(parameterCount - 2);
        if (node instanceof SOAPElement) {
            SOAPElement element = (SOAPElement) node;
            String paramName = element.getLocalName();
            String paramValue = element.getValue();
            parameters.append(paramName).append("=").append(paramValue);
        }
        return parameters.toString();
    }

}
