package edujin.repository;

import edujin.model.Subscription;
import edujin.util.HibernateUtil;
import io.github.cdimascio.dotenv.Dotenv;
import org.hibernate.Session;
import org.hibernate.Transaction;

import java.time.LocalDateTime;

public class SubscriptionRepository extends BaseRepository {
    final protected long expiryDurationHours = Long.parseLong(Dotenv.load().get("SUBSCRIPTION_EXPIRE_HOURS"));
    final protected long expiryDurationMinutes = Long.parseLong(Dotenv.load().get("SUBSCRIPTION_EXPIRE_MINUTES"));
    private static SubscriptionRepository instance = null;
    private SubscriptionRepository() {
        super();
    }
    public static SubscriptionRepository getInstance() {
        if (instance == null) {
            instance = new SubscriptionRepository();
        }
        return instance;
    }
    public LocalDateTime generateExpiryDate() {
        return LocalDateTime.now().plusHours(expiryDurationHours).plusMinutes(expiryDurationMinutes);
    }

    public String create(String idUser) {
        Transaction tx = null;
        Session session = HibernateUtil.getSessionFactory().getCurrentSession();
        try {
            tx = session.beginTransaction();
            Subscription subscription = session.find(Subscription.class, idUser);
            if (subscription == null) {
                subscription = new Subscription();
                subscription.setIdUser(idUser);
            }
            subscription.setExpiryDate(generateExpiryDate());

            session.save(subscription);
            tx.commit();
            System.out.println("Subscription created");
            return "200";
        } catch (Exception e) {
            if (tx != null) {
                tx.rollback();
            }
            e.printStackTrace();
            throw e;
        }
    }
    public String status(String idUser) {
        Transaction tx = null;
        Session session = HibernateUtil.getSessionFactory().getCurrentSession();
        try {
            tx = session.beginTransaction();
            Subscription subscription = session.find(Subscription.class, idUser);
            if (subscription == null) {
                throw new RuntimeException("Not Subscribed");
            }
            LocalDateTime expiryDate = subscription.getExpiryDate();
            if (LocalDateTime.now().isAfter(expiryDate)) {
                session.delete(subscription);
                tx.commit();
                throw new RuntimeException("Not Subscribed");
            }
            tx.commit();
            return "200";
        } catch (Exception e) {
            if (tx != null) {
                tx.rollback();
            }
            e.printStackTrace();
            return e.getMessage();
        }
    }

    public void delete(String username) {
        Transaction tx = null;
        Session session = HibernateUtil.getSessionFactory().getCurrentSession();
        try {
            tx = session.beginTransaction();
            Subscription subscription = session.find(Subscription.class, username);
            if (subscription == null) {
                return;
            }
            session.delete(subscription);
            tx.commit();
        } catch (Exception e) {
            if (tx != null) {
                tx.rollback();
            }
            throw e;
        }
    }
}
