package edujin.repository;

import edujin.model.Token;
import edujin.util.HibernateUtil;
import edujin.util.SecurityUtil;
import org.hibernate.Session;
import org.hibernate.Transaction;

import java.io.UnsupportedEncodingException;
import java.time.LocalDateTime;

import static edujin.util.SecurityUtil.generateRandomString;

abstract public class TokenRepository extends BaseRepository {
    protected long expiryDurationHours;
    protected long expiryDurationMinutes;
    public LocalDateTime generateExpiryDate() {
        return LocalDateTime.now().plusHours(expiryDurationHours).plusMinutes(expiryDurationMinutes);
    }

    public String create(String userId, String userEmail) throws UnsupportedEncodingException {
        String newToken = generateRandomString(32);
        Transaction tx = null;
        Session session = HibernateUtil.getSessionFactory().getCurrentSession();
        try {
            tx = session.beginTransaction();
            Token token = session.find(Token.class, userId);
            if (token == null) {
                token = new Token();
                token.setUserId(userId);
            }
            token.setHashedToken(SecurityUtil.hashPassword(newToken));
            token.setExpiryDate(generateExpiryDate());

            session.save(token);
            tx.commit();
            sendEmail(userEmail, newToken);
            return "200";
        } catch (Exception e) {
            if (tx != null) {
                tx.rollback();
            }
            e.printStackTrace();
            throw e;
        }
    }

    public void delete(String userId) {
        Transaction tx = null;
        Session session = HibernateUtil.getSessionFactory().getCurrentSession();
        try {
            tx = session.beginTransaction();
            Token token = session.find(Token.class, userId);
            if (token == null) {
                throw new RuntimeException("token not found");
            }
            session.delete(token);
            tx.commit();
        } catch (Exception e) {
            if (tx != null) {
                tx.rollback();
            }
            throw e;
        }
    }

    public String validateToken(String userId, String inputToken) {
        System.out.println("userId: " + userId);
        System.out.println("inputToken: " + inputToken);
        Transaction tx = null;
        Session session = HibernateUtil.getSessionFactory().getCurrentSession();
        try {
            tx = session.beginTransaction();
            Token token = session.find(Token.class, userId);
            if (token == null) {
                throw new RuntimeException("token not found");
            }
            LocalDateTime expiryDate = token.getExpiryDate();
            if (LocalDateTime.now().isAfter(expiryDate)) {
                session.delete(token);
                tx.commit();
                throw new RuntimeException("token expired");
            }
            String hashedSaltedToken = token.getHashedToken();
            if (!SecurityUtil.checkPassword(inputToken, hashedSaltedToken)) {
                throw new RuntimeException("token not valid");
            }
            tx.commit();
//            delete(userId);
            return "200";
        } catch (Exception e) {
            if (tx != null) {
                tx.rollback();
            }
            e.printStackTrace();
            return e.getMessage();
        }
    }

    public abstract void sendEmail(String userEmail, String newToken) throws UnsupportedEncodingException;
}
